uniform mat4 g_WorldViewProjectionMatrix;
uniform float g_Time;

uniform sampler2D m_NoiseMap;

uniform float m_Speed;

attribute vec3 inPosition;
attribute vec4 inColor;
attribute vec4 inTexCoord;
attribute vec3 inNormal;

varying vec4 color;

#ifdef USE_TEXTURE
varying vec4 texCoord;
#endif

#ifdef POINT_SPRITE
uniform mat4 g_WorldViewMatrix;
uniform mat4 g_WorldMatrix;
uniform vec3 g_CameraPosition;
uniform float m_Quadratic;
const float SIZE_MULTIPLIER = 30.0;
//attribute float inSize;
#endif

uniform float m_Alpha;

// The same wind noise that grass uses for now
// ... with some tweaks
float simpleNoise( float x, float y, float z )
{
    // We'll cheat and just adjust x by a little bit of
    // z and y by a lot bit of z and use them as a texture look-up.
    //x += z * 0.1;
    //y += z;
    //x = mod(x, 64.0) / 64.0;
    //y = mod(y, 64.0) / 64.0;
    //z = z / 64.0;
    x += z; // * 0.001;
    y += z * 0.1;
    x = mod(x * 0.05, 1.0);
    y = mod(y * 0.05, 1.0);
    
    return texture2D( m_NoiseMap, vec2(x,y) ).r - 0.5;
}

float simpleNoise( float x, float y, float z, float time )
{
    // Move x,y to be in a nicer range
    x = x * 0.3;
    z = z * 0.3;
    
    x = mod(x, 1.0);
    z = mod(z, 1.0);
    
    return texture2D( m_NoiseMap, vec2(x + time, z + y) ).r;
}

void main(){
    vec4 pos = vec4(inPosition, 1.0);
    
    float projected = inColor.y;
    float len = inColor.x;
    float startSize = inColor.z;
    float endSize = inColor.a;

    projected = projected + (g_Time / len) * m_Speed;
    projected = mod( projected, 1.0 ); 
    
    pos.xyz += inNormal * projected * len;
    
    // mix( startSize, endSize, normalized projection )
    float part = projected; // / inColor.x; 
    float size = 5.0 * mix( startSize, endSize, part );
    
    float scale = length( g_WorldMatrix[0].xyz );
    
    vec4 worldPos = g_WorldMatrix * pos;
    float d = distance(g_CameraPosition.xyz, worldPos.xyz);


    gl_Position = g_WorldViewProjectionMatrix * pos;
    
    float cutoff = 0.3;
    //float curve = max( cutoff - part, part - (1.0 + cutoff) );
    float curve = max( cutoff - part, part - (1.0 - cutoff) );
    // returns 0 if curve <= 0
    //curve = step( 0.0, curve ) * 5 * curve;
    curve = step( 0.0, curve ) * (1.0 / 0.3) * curve;
    float alpha = 0.75 - (curve * curve); 
//    float alpha = 0.6 - (curve * curve); 

    //float curve = abs(0.5 - part) * 2;     
    //float alpha = 0.5 - ((curve * curve * curve * curve) * 0.3);
    //float alpha = 0.5; 
    //color = vec4(0.9,0.9,1.0,alpha); 
    color = vec4(1.0,1.0,1.0,alpha); 


    texCoord = inTexCoord;
                
    gl_PointSize = max(1.0, ((size * SIZE_MULTIPLIER * m_Quadratic) / d) * scale );
}


